"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var match_1 = require("../translate/match");
var printf_1 = require("../translate/printf");
var Fallback;
(function (Fallback) {
    Fallback[Fallback["throw"] = 0] = "throw";
    Fallback[Fallback["id"] = 1] = "id";
    Fallback[Fallback["skip"] = 2] = "skip";
})(Fallback = exports.Fallback || (exports.Fallback = {}));
var initial_options = {
    datas: {},
    fallback: Fallback.throw,
    start_file: 'stat_descriptions'
};
var formatStats = function (stats, options) {
    if (options === void 0) { options = {}; }
    var _a = Object.assign({}, initial_options, options), datas = _a.datas, fallback = _a.fallback, start_file = _a.start_file;
    // translated lines
    var lines = [];
    // array of stat_ids for which hash lookup failed
    var untranslated = new Map(stats.map(function (stat) { return [stat.id, stat]; }));
    var description_file = datas[start_file];
    var _loop_1 = function () {
        var data = description_file.data;
        lines.push.apply(lines, formatWithFinder(untranslated, function (_a) {
            var id = _a.id;
            return data[id];
        }));
        lines.push.apply(lines, formatWithFinder(untranslated, function (_a) {
            var id = _a.id;
            return findDescription(id, data);
        }));
        description_file = description_file.meta.include
            ? datas[description_file.meta.include]
            : undefined;
    };
    while (description_file !== undefined) {
        _loop_1();
    }
    lines.push.apply(lines, formatWithFallback(untranslated, fallback));
    return lines;
};
exports.default = formatStats;
/**
 * O(n) lookup if hash lookup fails
 *
 * @param stat_id
 * @param locale_data
 */
function findDescription(stat_id, locale_data) {
    return Object.values(locale_data).find(function (_a) {
        var stats = _a.stats;
        return stats.includes(stat_id);
    });
}
var NO_DESCRIPTION = 'NO_DESCRIPTION';
// stats will get mutated
function formatWithFinder(stats, find) {
    var lines = [];
    var translated = new Set();
    for (var _i = 0, _a = Array.from(stats.entries()); _i < _a.length; _i++) {
        var _b = _a[_i], stat_id = _b[0], stat = _b[1];
        if (translated.has(stat_id)) {
            continue;
        }
        var description = find(stat);
        if (description !== undefined) {
            var translation = translate(description, stats);
            if (translation === undefined) {
                throw new Error("matching translation not found for '" + stat.id + "'");
            }
            else {
                // mark as translated
                description.stats.forEach(function (translated_id) {
                    stats.delete(translated_id);
                    translated.add(translated_id);
                });
                if (translation === NO_DESCRIPTION) {
                    lines.push(stat_id + " (hidden)");
                }
                else {
                    lines.push(translation);
                }
            }
        }
    }
    return lines;
}
function translate(description, provided) {
    var stats = description.stats, no_description = description.no_description, translations = description.translations;
    if (no_description === true) {
        return NO_DESCRIPTION;
    }
    // intersect the required stat_ids from the desc with the provided
    var required_stats = stats
        .map(function (stat_id) {
        var stat = provided.get(stat_id);
        // default the value to 0
        if (stat === undefined) {
            return {
                id: stat_id,
                value: 0
            };
        }
        else {
            return stat;
        }
    })
        .filter(function (stat) { return stat !== null; });
    var translation = matchingTranslation(translations, required_stats);
    if (translation === undefined) {
        return undefined;
    }
    else {
        return printf_1.default(translation.text, required_stats.map(function (_a) {
            var value = _a.value;
            return value;
        }), translation.formatters);
    }
}
function matchingTranslation(translations, stats) {
    var args = stats.map(function (_a) {
        var value = _a.value;
        return value;
    });
    return translations.find(function (translation) {
        return match_1.matches(args, translation.matchers).every(function (match) { return match === match_1.Match.subset || match === match_1.Match.exact; });
    });
}
function formatWithFallback(stats, fallback) {
    if (fallback === Fallback.throw) {
        if (stats.size > 0) {
            throw new Error('no descriptions found for ' + Array.from(stats.keys()).join(','));
        }
        else {
            return [];
        }
    }
    else if (fallback === Fallback.id) {
        return Array.from(stats.keys());
    }
    else if (fallback === Fallback.skip) {
        return [];
    }
    else if (typeof fallback === 'function') {
        return Array.from(stats.entries())
            .map(function (_a) {
            var id = _a[0], stat = _a[1];
            return fallback(id, stat);
        })
            .filter(function (line) { return typeof line === 'string'; });
    }
    else {
        // should ts recognize that this is unreachable code? enums can prob
        // be extended at runtime an therfore somebody could mess with them
        throw new Error("unrecognized fallback type '" + fallback + "'");
    }
}
